import enumerate.Action;
import enumerate.State;
import gameInterface.AIInterface;



public final class RatedAct extends Act implements Comparable<RatedAct> {
	final int damage;
	int successes = 1;
	int uses = 1;
	boolean priorKnowledge = false;
	int thisGameSuccess = 1;
	int thisGameUses = 1;
	
	public RatedAct(Act act, int damage)
	{
		super(act);
		this.damage = damage;
	}
	
	public RatedAct(Act act, int damage, int successes)
	{
		this(act,damage,successes,1);
	}
	public RatedAct(Act act, int damage, int successes, int uses)
	{
		super(act);
		this.damage = damage;
		this.successes = successes;
		this.uses = uses;
	}
	
	public boolean successful()
	{
		return damage != 0;
	}
	
	public static RatedAct create(String s, int version)
	{
		String[] args = s.split(":");
		int x = 0;
		String myCharacter = AIInterface.CHARACTER_ZEN;
		String opCharacter = AIInterface.CHARACTER_ZEN;
		if(isCharacter(args[1]))
		{
			myCharacter = args[1];
			opCharacter = args[2];
			x = 2;
		}
		int successRate = Integer.valueOf(args[x+1]);
		int uses = Integer.valueOf(args[x+2]);
		Action myAction = Action.valueOf(args[x+3]);
		Action opAction = Action.valueOf(args[x+4]);
		int dist = Integer.valueOf(args[x+5]);
		int hitboxWidth = 0;
		if(version >= 2)
		{
			hitboxWidth = Integer.valueOf(args[x+6]);
			x++;
		}
		int energyCost = 0;
		if(version >= 3)
		{
			energyCost = Integer.valueOf(args[x+6]);
			x++;
		}
		State myState = State.valueOf(args[x+6]);
		State opState = State.valueOf(args[x+7]);
		int success = Integer.valueOf(args[x+8]);
		Act lastMyAct = null;
		Act lastOpAct = null;
		if(!args[x+9].startsWith("<null>"))
			lastMyAct = Act.create(Act.getSubAct(s,0),version);
		if(!args[x+10].startsWith("<null>"))
			lastOpAct = Act.create(Act.getSubAct(s,1),version);
		
		Act cre = new Act(myAction, opAction, dist, myState, opState, hitboxWidth, energyCost);
		cre.lastMyAct = lastMyAct; cre.lastOpAct = lastOpAct;
		cre.myCharacter = myCharacter; cre.opCharacter = opCharacter;
		
		if(uses < successRate)
			uses = successRate;
		return new RatedAct(cre,success,successRate,uses);
	}
	
	public String toString()
	{
		return "RatedAct:"+myCharacter+":"+opCharacter+":"+successes+":"+uses+":"+myAction.name()+":"+opAction.name()+":"+dist+":"+hitboxWidth+":"+energyCost+":"+myState.name()+":"+opState.name()+":"+damage+":<"+lastMyAct+">:<"+lastOpAct+">";
	}

	public int successes()
	{
		if(thisGameUses >= LearningControl.thisGameUseThreshold)
			return thisGameSuccess;
		return successes;
	}
	
	public int uses()
	{
		if(thisGameUses >= LearningControl.thisGameUseThreshold)
			return thisGameUses;
		return uses;
	}
	
	@Override
	public int compareTo(RatedAct a) {
		if(priorKnowledge)
		{
			if(!a.priorKnowledge)
				return 1;
		}
		else if(a.priorKnowledge)
			return -1;
		//return a.successRate - successRate + a.success - success;
		return (int)((a.successes()/(double)a.uses() - successes()/(double)uses())*50 + a.damage - damage); //success|damage difference much better
	}
}
