package rule;

/**
 * 
 * @author Kevin Majchrzak
 * 
 */
public abstract class Rule implements Comparable<Rule> {

	protected GameStateInterface gameState;
	public boolean isPermanentCombo;

	public Rule(GameStateInterface gameState) {
		this(gameState, false);
	}

	public Rule(GameStateInterface gameState, boolean isPermanentCombo) {
		this.gameState = gameState;
		this.isPermanentCombo = isPermanentCombo;
	}

	/**
	 * Returns true, if the rule will perform well in the current situation.
	 * 
	 * @return true, if the rule will perform well in the current situation
	 */
	public abstract boolean satisfied();

	/**
	 * Executes the rule by calling a command.
	 */
	public abstract void execute();

	/**
	 * rules with lower priority will be executed first
	 * 
	 * @return priority
	 */
	public abstract int getPriority();

	@Override
	public int compareTo(Rule other) {
		return this.getPriority() - other.getPriority();
	}

}
