package rule;

import enumerate.State;

/**
 * 
 * @author Kevin Majchrzak
 * 
 */
public class ShortSlideCounter extends Rule {

	public ShortSlideCounter(GameStateInterface gameState) {
		super(gameState);
	}

	@Override
	public boolean satisfied() {
		// no need for defense
		if (!gameState.getEnemyCharacterData().getAction().name()
				.equals("STAND_D_DB_BB")) {
			return false;
		}

		boolean myCharacterOnLeftSide = gameState.getCommandCenter().getMyX() < gameState
				.getCommandCenter().getEnemyX();

		// enemy is attacking in wrong direction
		if ((gameState.getEnemyCharacterData().isFront() && (myCharacterOnLeftSide))
				|| ((!gameState.getEnemyCharacterData().isFront()) && (!myCharacterOnLeftSide))) {
			return false;
		}

		// check if my character is in enemy attack range
		int enemyAttackNearestHitBoxEdge; // attack hitbox
		int myNearestHitBoxEdge; // character hitbox

		if (myCharacterOnLeftSide) {
			// see Attack.materialise
			enemyAttackNearestHitBoxEdge = gameState.getEnemyCharacterData()
					.getX()
					+ 256
					- gameState.getEnemyMotionData().getAttackHit().getR();
			myNearestHitBoxEdge = gameState.getMyCharacterData().getRight();
		} else {
			enemyAttackNearestHitBoxEdge = gameState.getEnemyCharacterData()
					.getX()
					+ gameState.getEnemyMotionData().getAttackHit().getR();
			myNearestHitBoxEdge = gameState.getMyCharacterData().getLeft();
		}

		// distance between my character and enemy attack hitbox
		int distX = Math
				.abs(enemyAttackNearestHitBoxEdge - myNearestHitBoxEdge);

		// remaining distance enemy will travel with slide
		int absSpeedX = Math.abs(gameState.getEnemyCharacterData().getSpeedX());
		int restDistX = (absSpeedX * (absSpeedX + 1)) / 2; // Gau

		int delay = 15;
		int nowRemainingFrame = gameState.getEnemyCharacterData()
				.getRemainingFrame() - delay;

		// return true if enemy will come to halt in near range and timing is
		// good
		return (distX > restDistX) && ((distX - restDistX) < 300)
				&& (nowRemainingFrame <= 20) && (nowRemainingFrame > 10)
				&& (ActiveComboRuleWrapper.getSingleton().rule == null);
	}

	@Override
	public void execute() {

		if (gameState.getMyCharacterData().getState() != State.AIR) {
			// counter from ground
			// slide
			gameState.getCommandCenter().commandCall("2 1 4 _ B");
		} else {
			// counter from air
			// air slide will either hit enemy or flee
			gameState.getCommandCenter().commandCall("AIR_D_DF_FB");
		}

	}

	@Override
	public int getPriority() {
		return -3;
	}

}
